#!/usr/bin/env python
# 
# Use stub mode with google public DNS as recursive resolvers
# to query addresses associated with a host.
#

import getdns, sys, pprint

google_public_dns = [
    {'address_data': '8.8.8.8', 'address_type': 'IPv4'},
    {'address_data': '8.8.4.4', 'address_type': 'IPv4'},
    {'address_data': '2001:4860:4860::8888', 'address_type': 'IPv6'},
    {'address_data': '2001:4860:4860::8844', 'address_type': 'IPv6'},
]

hostname = sys.argv[1]

ctx = getdns.Context()
ctx.resolution_type = getdns.RESOLUTION_STUB
ctx.upstream_recursive_servers = google_public_dns

extensions = { "return_both_v4_and_v6" : getdns.EXTENSION_TRUE }

try:
    results = ctx.address(name=hostname, extensions=extensions)
except getdns.error as e:
    print(str(e))
    sys.exit(1)

if results.status == getdns.RESPSTATUS_GOOD:
    for addr in results.just_address_answers:
        print(addr["address_data"])
else:
    print("getdns.address() returned an error: %d".format(results.status))




#!/usr/bin/env python
#

"""
Lookup an NS record and printout all the hostnames and associated IP
addresses of the listed nameservers.
"""

import getdns, pprint, sys

extensions = { "return_both_v4_and_v6" : getdns.EXTENSION_TRUE }


def usage():
    print("""Usage: get-ns-ip.py <zone>

where <zone> is a DNS zone (domain).
""")
    sys.exit(1)


def get_ip(ctx, qname):
    iplist = []
    try:
        results = ctx.address(name=qname, extensions=extensions)
    except getdns.error as e:
        print(str(e))
        sys.exit(1)

    if results.status == getdns.RESPSTATUS_GOOD:
        for addr in results.just_address_answers:
            iplist.append(addr['address_data'])
    else:
        print("getdns.address() returned an error: {0}".format(results['status']))
    return iplist


if __name__ == '__main__':

    if len(sys.argv) != 2:
        usage()

    qname = sys.argv[1]

    ctx = getdns.Context()
    try:
        results = ctx.general(name=qname, request_type=getdns.RRTYPE_NS)
    except getdns.error as e:
        print(str(e))
        sys.exit(1)
    status = results.status

    hostlist = []
    if status == getdns.RESPSTATUS_GOOD:
        for reply in results.replies_tree:
            answers = reply['answer']
            for answer in answers:
                if answer['type'] == getdns.RRTYPE_NS:
                    iplist = get_ip(ctx, answer['rdata']['nsdname'])
                    for ip in iplist:
                        hostlist.append( (answer['rdata']['nsdname'], ip) )
    elif status == getdns.RESPSTATUS_NO_NAME:
        print("{0}: no such DNS zone".format(qname))
    elif status == getdns.RESPSTATUS_ALL_TIMEOUT:
        print("{0}, NS: query timed out".format(qname))
    else:
        print("{0}, NS: unknown return code: {1}".format(qname, results["status"]))

    # Print out each NS server name and IP address
    for (nsdname, addr) in sorted(hostlist):
        print(nsdname, addr)




# an example of using getdns to pull out a TLSA record,
# extract a certificate, extract the public key, and then
# encrypt some text
#
# requires the following Python modules:
#    getdns
#    m2crypto
#


import getdns
import M2Crypto as m2
from M2Crypto import RSA
import sys



# 
# I commented out the "return None" because this is demo code and you
# should be able to play with it.  But, in deployed applications you
# MUST check that dnssec_status is DNSSEC_SECURE
#

def get_first_secure_response(results):
    replies_tree = results.replies_tree
    if (not replies_tree) or (not len(replies_tree)) or (not replies_tree[0]['answer']) or (not len(replies_tree[0]['answer'])):
        print 'empty answer list'
        return None
    else:
        reply = replies_tree[0]
        if reply['dnssec_status'] != getdns.DNSSEC_SECURE:
            print 'insecure reply'
#            return None                      
        answer = replies_tree[0]['answer']
        record = [ x for x in answer if x['type'] is getdns.RRTYPE_TLSA ]
        if len(record) == 0:
            print 'no answers of type TLSA'
            return None
        return record[0]
    
def main():
    tls_name = '77fa5113ab6a532ce2e6901f3bd3351c0db5845e0b1b5fb09907808d._smimecert.getdnsapi.org'

    if len(sys.argv) == 2:
        tls_name = sys.argv[1]
    c = getdns.Context()
    extensions = { 'dnssec_return_status' : getdns.EXTENSION_TRUE }
    results = c.general(tls_name, request_type=getdns.RRTYPE_TLSA, extensions=extensions)
    if results.replies_full['status'] != getdns.RESPSTATUS_GOOD:
        print 'query status is {0}'.format(results.status)
        sys.exit(1)
    else:
        record = get_first_secure_response(results)
        cert = record['rdata']['certificate_association_data']
        try:
            x509 = m2.X509.load_cert_der_string(cert)
            rsakey = x509.get_pubkey().get_rsa()
            encrypted = rsakey.public_encrypt("A chunk of text", RSA.pkcs1_oaep_padding)
            print encrypted.encode('base64')
        except:
            print 'Error: ', sys.exc_info()[0]
            sys.exit(1)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
#
# Validate a TLS certificate with DANE-EE usage.
# Get a TLS certificate from a HTTP server and verify it with
# DANE/DNSSEC. Only supports TLSA usage=3 (DANE-EE)
# 

import os.path, sys, socket, hashlib
from M2Crypto import SSL, X509
import getdns


def usage():
    print """\
Usage: %s [hostname] [port]\
""" % os.path.basename(sys.argv[0])
    sys.exit(1)


def compute_hash(func, string):
    """compute hash of string using given hash function"""
    h = func()
    h.update(string)
    return h.hexdigest()


def get_addresses(hostname):

    extensions = {
        "return_both_v4_and_v6" : getdns.EXTENSION_TRUE
    }
    ctx = getdns.Context()
    try:
        results = ctx.address(name=hostname, extensions=extensions)
    except getdns.error, e:
        print(str(e))
        sys.exit(1)

    status = results.status

    address_list = []
    if status == getdns.RESPSTATUS_GOOD:
        for addr in results.just_address_answers:
            address_list.append((addr['address_type'], addr['address_data']))
    else:
        print "getdns.address(): failed, return code: %d" % status

    return address_list


def get_tlsa_rdata_set(replies, requested_usage=None):
    tlsa_rdata_set = []
    for reply in replies:
        for rr in reply['answer']:
            if rr['type'] == getdns.RRTYPE_TLSA:
                rdata = rr['rdata']
                usage = rdata['certificate_usage']
                selector = rdata['selector']
                matching_type = rdata['matching_type']
                cadata = rdata['certificate_association_data']
                cadata = str(cadata).encode('hex')
                if usage == requested_usage:
                    tlsa_rdata_set.append(
                        (usage, selector, matching_type, cadata) )
    return tlsa_rdata_set


def get_tlsa(port, proto, hostname):

    extensions = {
        "dnssec_return_only_secure" : getdns.EXTENSION_TRUE,
    }
    qname = "_%d._%s.%s" % (port, proto, hostname)
    ctx = getdns.Context()
    try:
        results = ctx.general(name=qname,
                              request_type=getdns.RRTYPE_TLSA,
                              extensions=extensions)
    except getdns.error, e:
        print(str(e))
        sys.exit(1)

    status = results.status

    if status == getdns.RESPSTATUS_GOOD:
        return get_tlsa_rdata_set(results.replies_tree, requested_usage=3)
    else:
        print "getdns.general(): failed, return code: %d" % status
        return None


def verify_tlsa(cert, usage, selector, matchtype, hexdata1):

    if selector == 0:
        certdata = cert.as_der()
    elif selector == 1:
        certdata = cert.get_pubkey().as_der()
    else:
        raise ValueError("selector type %d not recognized" % selector)

    if matchtype == 0:
        hexdata2 = hexdump(certdata)
    elif matchtype == 1:
        hexdata2 = compute_hash(hashlib.sha256, certdata)
    elif matchtype == 2:
        hexdata2 = compute_hash(hashlib.sha512, certdata)
    else:
        raise ValueError("matchtype %d not recognized" % matchtype)

    if hexdata1 == hexdata2:
        return True
    else:
        return False


if __name__ == '__main__':

    try:
        hostname, port = sys.argv[1:]
        port = int(port)
    except:
        usage()

    tlsa_rdata_set = get_tlsa(port, "tcp", hostname)

    for (iptype, ipaddr) in get_addresses(hostname):

        print "Connecting to %s at address %s ..." % (hostname, ipaddr)
        ctx = SSL.Context()

        if iptype == "IPv4":
            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        elif iptype == "IPv6":
            sock = socket.socket(socket.AF_INET6, socket.SOCK_STREAM)
        else:
            raise ValueError, "Unknown address type: %s" % iptype

        sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        connection = SSL.Connection(ctx, sock=sock)

        # set TLS SNI extension if available in M2Crypto on this platform
        # Note: the official M2Crypto release does not yet (as of late 2014)
        # have support for SNI, sigh, but patches exist.
        try:
            connection.set_tlsext_host_name(hostname)
        except AttributeError:
            pass

        # Per https://tools.ietf.org/html/draft-ietf-dane-ops, for DANE-EE
        # usage, certificate identity checks are based solely on the TLSA 
        # record, so we ignore name mismatch conditions in the certificate.
        try:
            connection.connect((ipaddr, port))
        except SSL.Checker.WrongHost:
            pass

        chain = connection.get_peer_cert_chain()
        cert = chain[0]

        # find a matching TLSA record entry for the certificate
        tlsa_match = False
        for (usage, selector, matchtype, hexdata) in tlsa_rdata_set:
            if verify_tlsa(cert, usage, selector, matchtype, hexdata):
                tlsa_match = True
                print "Matched TLSA record %d %d %d %s" % \
                    (usage, selector, matchtype, hexdata)
            else:
                print "Didn't match TLSA record %d %d %d %s"% \
                    (usage, selector, matchtype, hexdata)

        if not tlsa_match:
            print "No Matching DANE-EE TLSA record found."

        connection.close()
        ctx.close()




#
# Note that we expect that each pin in the pinset list
#   must be prefaced with pin-<algorithm>= and the pin 
#   itself must be base64-encoded and enclosed in double-
#   quotes.  We may loosen this up in a future version
#

import getdns
c = getdns.Context()
u = [ { 'address_data': '185.49.141.37', 
        'address_type': 'IPv4', 
        'tls_pubkey_pinset': ['pin-sha256="foxZRnIh9gZpWnl+zEiKa0EJ2rdCGroMWm02gaxSc9S="']
 }]
c.resolution_type = getdns.RESOLUTION_STUB
c.dns_transport_list = [ getdns.TRANSPORT_TLS ]
c.upstream_recursive_servers = u
f = c.address('getdnsapi.net')



#!/usr/bin/env python
# coding=utf-8

import getdns, sys

try:
    ulabel = getdns.alabel_to_ulabel('xn--p1acf')
    # Next line contains a utf-8 string
    alabel = getdns.ulabel_to_alabel('рус')
    ulabel1 = getdns.alabel_to_ulabel('xn--vermgensberatung-pwb')
    # Next line contains a utf-8 string
    alabel1 = getdns.ulabel_to_alabel('vermögensberatung')
except getdns.error as e:
    print(str(e))
    sys.exit(1)

print (ulabel)
print (alabel)
print (ulabel1)
print (alabel1)



import getdns
from struct import pack, unpack
    

def main():
    CLIENT_SUBNET_OPCODE = 8
        
    address = '192.168.1.0'
    host = 'getdnsapi.net'
    source_len = 12
    
    family = pack("!H", 1)  # start building the edns option fields
    source_len = pack('!B', source_len)
    scope_len = pack('!B', 0)

#
# encoding the binary data in strings makes it really easy
# to build packets by concatenating those strings
#
    address = pack('!BBBB', 192, 168, 1, 0)
    payload = family + source_len + scope_len + address
    ext = { 'add_opt_parameters': {'options':
                                       [ {'option_code': CLIENT_SUBNET_OPCODE,
                                          'option_data': payload} ] }}

    c = getdns.Context()
    c.resolution_type = getdns.RESOLUTION_STUB
    response = c.address(host, extensions=ext)

# do things with response ...
    print response

if __name__ == '__main__':
    main()



#!/usr/bin/env python
#

"""
get-ip.py: resolve given DNS names into IP addresses. The -s switch
constains answers to only ones secured by DNSSEC. The -4 switch only
returns IPv4 addresses, the -6 switch only IPv6 addresses.

An example run:

    $ get-ip.py -s www.huque.com www.google.com
    www.huque.com: IPv4  50.116.63.23
    www.huque.com: IPv6  2600:3c03:e000:81::a
    www.google.com: No DNSSEC secured responses found

"""

import getdns, sys, getopt

def usage():
    print("""\
Usage: get-ip.py [-s] [-4|-6] <domain1> <domain2> ...

    -s: only return DNSSEC secured answers
    -4: only return IPv4 address answers
    -6: only return IPv6 address answers

-4 and -6 are mutually exclusive. If both are specified, IPv6 wins.
""")
    sys.exit(1)

try:
    (options, args) = getopt.getopt(sys.argv[1:], 's46')
except getopt.GetoptError:
    usage()
else:
    if not args:
        usage()

extensions = { "return_both_v4_and_v6" : getdns.EXTENSION_TRUE }
desired_addr_type = None

for (opt, optval) in options:
    if opt == "-s":
        extensions["dnssec_return_only_secure"] = getdns.EXTENSION_TRUE
    elif opt == "-4":
        desired_addr_type = "IPv4"
    elif opt == "-6":
        desired_addr_type = "IPv6"

ctx = getdns.Context()

for hostname in args:
    try:
        results = ctx.address(name=hostname, extensions=extensions)
    except getdns.error as e:
        print(str(e))
        break
    status = results.status
    if status == getdns.RESPSTATUS_GOOD:
        for addr in results.just_address_answers:
            addr_type = addr['address_type']
            addr_data = addr['address_data']
            if (desired_addr_type == None) or (addr_type == desired_addr_type):
                print("{0}: {1}  {2}".format(hostname, addr_type, addr_data))
    elif status == getdns.RESPSTATUS_NO_SECURE_ANSWERS:
        print("{0}: No DNSSEC secured responses found".format(hostname))
    else:
        print("{0}: getdns.address() returned error: {1}".format(hostname, status))




#!/usr/bin/env python
#

"""
simple.py

A simple example to query a domain name and print out addresses
associated with it.
"""

import sys, getdns

hostname = sys.argv[1]

ctx = getdns.Context()
extensions = { 'return_both_v4_and_v6' : getdns.EXTENSION_TRUE }

try:
    results = ctx.address(name=hostname, extensions=extensions)
except:
    e = sys.exc.info()[0]
    print(str(e))
    sys.exit(1)

status = results.status

if status == getdns.RESPSTATUS_GOOD:
    for addr in results.just_address_answers:
        print (addr['address_data'])
else:
    print("{0}: getdns.address() returned error: {1}".format((hostname, status)))



#!/usr/bin/env python
#
# Given a DNS name and type, return the records in the DNS answer
# section only, excluding any RRSIG records.
#

import getdns, pprint, sys

extensions = { "dnssec_return_status" : getdns.EXTENSION_TRUE }

def get_rrtype(qtype):
    try:
        rrtype = eval("getdns.RRTYPE_%s" % qtype.upper())
    except AttributeError:
        print("Unknown DNS record type: {0}".format(qtype))
        sys.exit(1)
    else:
        return rrtype


def print_answer(r):
    pprint.pprint(r.replies_tree[0]['answer'])
    return


if __name__ == '__main__':

    qname, qtype = sys.argv[1:]
    rrtype = get_rrtype(qtype)

    ctx = getdns.Context()
    try:
        results = ctx.general(name=qname, request_type=rrtype,
                              extensions=extensions)
    except getdns.error as e:
        print(str(e))
        sys.exit(1)

    status = results.status

    if status == getdns.RESPSTATUS_GOOD:
        for reply in results.replies_tree:
            answers = reply['answer']           # list of 1 here
            for answer in answers:
                if answer['type'] != getdns.RRTYPE_RRSIG:
                    pprint.pprint(answer)
    elif status == getdns.RESPSTATUS_NO_NAME:
        print("{0}, {1}: no such name".format(qname, qtype))
    elif status == getdns.RESPSTATUS_ALL_TIMEOUT:
        print("{0}, {1}: query timed out".format(qname, qtype))
    else:
        print("{0}, {1}: unknown return code: {2}".format(qname, qtype, results.status))




#!/usr/bin/env python
#

"""
get-ip.py: resolve given DNS names into IP addresses. The -s switch
constains answers to only ones secured by DNSSEC. The -4 switch only
returns IPv4 addresses, the -6 switch only IPv6 addresses.

An example run:
$ python async-get-ip.py www.panix.com www.isoc.org www.verisignlabs.com

submitted query for www.panix.com
submitted query for www.isoc.org
submitted query for www.verisignlabs.com
www.panix.com: IPv4 166.84.62.125
www.panix.com: IPv4 166.84.62.253
www.verisignlabs.com: IPv4 72.13.58.64
www.verisignlabs.com: IPv6 2620:74:13:4400::201
www.isoc.org: IPv4 212.110.167.157
www.isoc.org: IPv6 2001:41c8:20::19

"""

import getdns, sys, getopt

def cbk(type, result, userarg, tid):
    if type == getdns.CALLBACK_COMPLETE:
        status = result.status
        if status == getdns.RESPSTATUS_GOOD:
            for addr in result.just_address_answers:
                addr_type = addr['address_type']
                addr_data = addr['address_data']
                print("{0}: {1} {2}".format(userarg, addr_type, addr_data))
        elif status == getdns.RESPSTATUS_NO_SECURE_ANSWERS:
            print("{0}: No DNSSEC secured responses found".format(hostname))
        else:
            print("{0}: getdns.address() returned error: {1}".format(hostname, status))
    elif type == getdns.CALLBACK_CANCEL:
        print('Callback cancelled')
    elif type == getdns.CALLBACK_TIMEOUT:
        print('Query timed out')
    else:
        print('Unknown error')


def usage():
    print("""\
Usage: get-ip.py [-s] [-4|-6] <domain1> <domain2> ...

    -s: only return DNSSEC secured answers
    -4: only return IPv4 address answers
    -6: only return IPv6 address answers

-4 and -6 are mutually exclusive. If both are specified, IPv6 wins.
""")
    sys.exit(1)

try:
    (options, args) = getopt.getopt(sys.argv[1:], 's46')
except getopt.GetoptError:
    usage()
else:
    if not args:
        usage()

extensions = { "return_both_v4_and_v6" : getdns.EXTENSION_TRUE }

ctx = getdns.Context()
tids = []
for hostname in args:
    try:
        tids.append(ctx.address(name=hostname, extensions=extensions, callback='cbk', userarg=hostname))
        print ('submitted query for {0}'.format(hostname))
    except getdns.error as e:
        print(str(e))
        break
ctx.run()



import getdns
import base64

u = [ { 'address_data': '185.49.141.37', 
        'address_type': 'IPv4', 
        'tsig_algorithm': 'hmac-md5.sig-alg.reg.int', 
        'tsig_name': 'hmac-md5.tsigs.getdnsapi.net',
        'tsig_secret':  base64.b64decode('16G69OTeXW6xSQ==')
 }]

c = getdns.Context()
c.resolution_type = getdns.RESOLUTION_STUB
c.upstream_recursive_servers = u
f = c.general('getdnsapi.net', request_type = getdns.RRTYPE_SOA)
print('tsig_status is {0}'.format(f.replies_tree[0]['tsig_status']))



# Copyright (c) 2014, NLnet Labs, Verisign, Inc.
# All rights reserved.
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#
# *  Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
# *  Neither the names of the copyright holders nor the
#    names of its contributors may be used to endorse or promote products
#    derived from this software without specific prior written permission.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Verisign, Inc. BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.



from distutils.core import setup, Extension
from distutils.fancy_getopt import fancy_getopt
import platform, os, sys

long_description = ( 'getdns is a set of wrappers around the getdns'
                     'library (http://www.getdnsapi.net), providing'
                     'Python language bindings for the API')

CFLAGS = [ '-g' ]
lib_dir = ""


if '--with-getdns' in sys.argv:
    getdns_root = sys.argv[sys.argv.index('--with-getdns')+1]
    inc_dir = getdns_root + '/include'
    lib_dir = getdns_root + '/lib'
    CFLAGS.append('-I{0}'.format(inc_dir))
    sys.argv.remove('--with-getdns')
    sys.argv.remove(getdns_root)

library_dirs = [ '/usr/local/lib' ]
if lib_dir:
    library_dirs.append(lib_dir)

platform_version = list(platform.python_version_tuple())[0:2]

if not ((platform_version[0] == '3') or (platform_version == ['2', '7'])):
    print('getdns requires Python version 2.7 or Python version 3.  Exiting ... ')
    os._exit(1)

getdns_module = Extension('getdns',
                    include_dirs = [ '/usr/local/include', ],
                    libraries = [ 'getdns' ],
                    library_dirs = library_dirs,
                    sources = [ 'getdns.c', 'pygetdns_util.c', 'context.c',
                                'context_util.c', 'result.c' ],
                    extra_compile_args = CFLAGS,
                    runtime_library_dirs = library_dirs,
                    )

setup(name='getdns',
      version='v1.0beta',
      description='Python bindings for getdns',
      long_description=long_description,
      license='BSD',
      author='Melinda Shore',
      author_email='melinda.shore@nomountain.net',
      url='http://getdns.readthedocs.org',
      dependency_links=['git+https://github.com/getdnsapi/getdns/tree/release/v0.9.0'],
      ext_modules = [ getdns_module ],
      classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        'Intended Audience :: System Administrators',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Topic :: Internet :: Name Service (DNS)',
        'Topic :: Security',
        'Topic :: Software Development :: Libraries :: Python Modules'
      ],
      keywords='DNS DNSSEC DANE',
)



#!/usr/bin/env python
#

"""
Lookup an SRV record and printout all the SRV priority, weight, targets, 
and associated IP addresses of the targets.
"""

import getdns, pprint, sys, time

srvname = sys.argv[1]

ctx = getdns.Context()
try:
    results = ctx.service(name=srvname)
except getdns.error as e:
    print(str(e))
    sys.exit(1)

if results.status == getdns.RESPSTATUS_GOOD:
    for reply in results.replies_tree:
        for a in reply["answer"]:
            rrname  = a["name"]
            rrtype  = a["type"]
            if rrtype == getdns.RRTYPE_SRV:
                rdata   = a["rdata"]
                prio, weight, port, target = rdata['priority'], rdata['weight'], rdata['port'], rdata['target']
                print("SRV {0} --> {1} {2} {3} {4}".format(rrname, prio, weight, port, target))
else:
    print("getdns.service() returned an error: {0}".format(results.status))



#!/usr/bin/env python

import getdns, sys

hostname = sys.argv[1]

ctx = getdns.Context()
ctx.resolution_type = getdns.RESOLUTION_STUB

extensions = { "return_both_v4_and_v6" : getdns.EXTENSION_TRUE }
ctx.resolution_type = getdns.RESOLUTION_STUB

try:
    results = ctx.address(name=hostname, extensions=extensions)
except getdns.error as e:
    print(str(e))
    sys.exit(1)

if results.status == getdns.RESPSTATUS_GOOD:
    for addr in results.just_address_answers:
        print(addr["address_data"])
else:
    print("getdns.address() returned an error: {0}".format(results["status"]))




#!/usr/bin/env python
#

"""
Lookup an MX record and printout all the MX preference, target, and
associated IP addresses of the targets.
"""

import getdns, pprint, sys

extensions = { "return_both_v4_and_v6" : getdns.EXTENSION_TRUE }


def get_ip(ctx, qname):
    iplist = []
    try:
        results = ctx.address(name=qname, extensions=extensions)
    except getdns.error as e:
        print(str(e))
        sys.exit(1)

    if results.status == getdns.RESPSTATUS_GOOD:
        for addr in results.just_address_answers:
            iplist.append(addr['address_data'])
    else:
        print("getdns.address() returned an error: {0}".format(results.status))
    return iplist


if __name__ == '__main__':

    qname = sys.argv[1]

    ctx = getdns.Context()
    try:
        results = ctx.general(name=qname, request_type=getdns.RRTYPE_MX)
    except getdns.error as e:
        print(str(e))
        sys.exit(1)

    status = results.status

    hostlist = []
    if status == getdns.RESPSTATUS_GOOD:
        for reply in results.replies_tree:
            answers = reply['answer']
            for answer in answers:
                if answer['type'] == getdns.RRTYPE_MX:
                    iplist = get_ip(ctx, answer['rdata']['exchange'])
                    for ip in iplist:
                        hostlist.append( (answer['rdata']['preference'], \
                                          answer['rdata']['exchange'], ip) )
    elif status == getdns.RESPSTATUS_NO_NAME:
        print("{0}, {1}: no such name".format(qname, qtype))
    elif status == getdns.RESPSTATUS_ALL_TIMEOUT:
        print("{0}, {1}: query timed out".format(qname, qtype))
    else:
        print("{0}, {1}: unknown return code: {2}".format(qname, qtype, results["status"]))

    for (pref, mx, addr) in sorted(hostlist):
        print(pref, mx, addr)
